import base64
import os
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
class using:
    def base(password:str=""):
        salt = os.urandom(16)
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=100000,
        )
        key = base64.urlsafe_b64encode(kdf.derive(password))
        f = Fernet(key)
        return f
    def encrypt(phrase:str,letter:str):
        b=using.base(phrase)
        f=b.encrypt(letter)
        return f.decode()
    def decrypt(phrase:str,letter:str):
        b=using.base(phrase)
        f=b.decrypt(letter)
        return f.decode()


from pytube import Playlist as ytPl, YouTube as yt
class Youtube:
    def getPlaylistInfo(playlistUrl:str):
        obj=ytPl(playlistUrl)
        OBJ={"playlist_ID":obj.playlist_id,"playlist_title":obj.title,"playlist_url":obj.playlist_url,"playlist_owner_id":obj.owner_id,"playlist_owner":obj.owner,"playlist_owner_url":obj.owner_url,"sidebar_info":obj.sidebar_info,"playlist_length":obj.length,"playlist_views":obj.views,"playlist_videos":obj.videos,"playlist_video_urls":obj.video_urls,"playlist_last_updated":obj.last_updated,"playlist_initial_data":obj.initial_data,"playlist_html":obj.html}
        return OBJ
    def downloadVideo(videoUrl:str, mode="video_highest_resolution", output_path=""):
        tt=yt(videoUrl)
        if mode=="video_highest_resolution": tt.streams.get_highest_resolution().download(output_path)
        elif mode=="video_lowest_resolution": tt.streams.get_lowest_resolution(output_path)
        elif mode=="audio_only": tt.streams.get_audio_only().download(output_path)
        else: raise Exception("Unknown mode: "+mode)
    def getVideoInfo(videoUrl:str):
        obj=yt(videoUrl)
        return obj.vid_info


from gtts import gTTS
import os
from playsound import playsound
def tts(text:str,axc:str,lng:str,u_dir:str):
    gTTS(text, "com" + axc, lng).save(f"{u_dir}/tmpSound.mp3")
    playsound(f"{u_dir}tmpSound.mp3")
    os.remove(f"{u_dir}tmpSound.mp3")


from .Base import *
class userData:
    def getUserObject(username:str):
        c=dict(ttapi.get_user("Bellapoarch"))
        return c
    def getDesc(obj:dict):
        e = userData.getseoProps(obj).get("metaParams").get("description")
        return e
    def getKeywords(obj:dict):
        e=userData.getseoProps(obj).get("metaParams").get("keywords")
        for i in e.split(","):
            yield i
    def getName(obj:dict):
        e = userData.getseoProps(obj).get("metaParams").get("name")
        return e
    def getseoProps(obj: dict):
        return obj.get("seoProps")
    def getUniqueID(obj:dict):
        obj.get("uniqueId")
    def getItems(obj:dict):
        return str(obj.items())


import multiprocessing,time,socket
from dataclasses import dataclass

class using:
    def create_process(target,args):
        p = multiprocessing.Process(target=target,args=args)
        return p
    def start_process(Process):
        global _mp_tkt_process
        Process.start()
        _mp_tkt_process=Process
# Q is the class for storing the variables used
class Q:
    urlSafeChars=".","\\","(",")","{","[","|","]","}","^",">","<"
class util:
    def calc(calc):
        exec(f'global __COMP_CALC_END__\n__COMP_CALC_END__=({calc})')
        return __COMP_CALC_END__
    def getip(S=""):
        return socket.gethostbyname(socket.gethostname())
    def do_for(n_seconds:int,func):
        t_end = time.time() + n_seconds
        while time.time() < t_end:func()
    def is_URLsafe(string:str,allowSub:bool=False):
        for i in Q.urlSafeChars:
            if i in string:return False
        if not allowSub:
            if "/" in string:return False
        return True
    def make_URLsafe(string:str,allowSub:bool=False,replaceChar:str="-"):
        e=string
        for i in Q.urlSafeChars:
            erg=e.replace(i, replaceChar)
            e=erg
        if not allowSub:e=e.replace("/",replaceChar)
        return e
    class Processing:
        @dataclass
        class tkt_process:
            process_object: multiprocessing.Process
            timeout: int
        def start(target, args=("",)):
            P = using.create_process(target, args)
            using.start_process(P)
        def close(S=""):
            _mp_tkt_process.join()


import json,requests
def weather(city, key):
    api_key = key
    uul = ("https://api.openweathermap.org/data/2.5/weather?q=" + city + "&appid=" + api_key)
    url = uul
    response = requests.get(url)
    data = json.loads(response.text)
    return data


__title__ = 'tkt-Toolkit'
__author__ = 'Suprime'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020-2021 Suprime'
__version__ = '1.5'

__tkt_OUS_ex__ = True
from . import Script_1,Script_2,Base,Crypto,SpeechRecongition,texttospeech,translator,weather,Youtube,tktUtil
from .errors import *

class TikTok:
    """
    Uses functionalities from the TikTokApi
    package and makes them more user friendly.
    """
    Music = Script_2.music
    User = Script_1.userData
    Base = Base

class SpeechRecognition:
    """
    Requires Pyaudio that you have to install
    using 'pipwin install pyaudio'.
    Records or reads audio and transforms it to text.
    Can also list Microphones.
    """
    def from_Microphone(show_msg: bool = True):
        SpeechRecongition.recon.recon_from_mic(show_msg)
    def from_Wav_File(wav_file: str, show_msg: bool = True):
        open(wav_file)
        SpeechRecongition.recon.recon_from_file(wav_file, show_msg)
    def list_Microphones(detailed: bool = True):
        for EE in SpeechRecongition.recon.get_mics(detailed): yield EE

class Info:
    def __version__(self=""):return __version__
    def __copyright__(self=""):return __copyright__
    def __license__(self=""):return __license__
    def __author__(self=""):return __author__

class Toolkit:
    class Crypto:
        """
        Uses the cryptography package to
        encrypt/decrypt 'Text' with 'Key'
        !! Does not work !!
        """
        def encrypt(Text:str,Key:str):
            raise OutOfService
            return Crypto.using.encrypt(Text.encode(),Key.encode())
        def decrypt(Text:str,Key:str):
            raise OutOfService
            return Crypto.using.decrypt(Text.encode(),Key.encode())
    def getIP(self=''):
        """
        Gets the users IP using the python
        built-in socket package.
        """
        return tktUtil.util.getip()
    def weather(city:str,open_weather_map_api_key:str):
        """
        Gets the weather from openweathermap.org,
        uses the standard free to use api.
        """
        weather.weather(city,open_weather_map_api_key)
    def texttospeech(Content:str, Lang:str, Axc:str="com",using_dir:str=''):
        """
        Creates a file in the 'using_dir' and
        plays it.
        """
        texttospeech.tts(Content,Axc,Lang,using_dir)
    def translate(Text:str,Dest,From='auto'):
        """
        Translate 'Text' from 'From' to 'Dest',
        set 'Dest' to "auto" if you don't know
        the language.
        """
        translator.Translate(Text,Dest,From)
tkt=Toolkit


from googletrans import Translator

translator = Translator()
class using:
    def translation_cut(Text:str):
        Text = str(Text)
        i = str(str(Text.replace("Translated(src=", "")))
        o, u, c, d, e = i.split(",")
        p = str(c.replace(" text=", ""))
        p = p.replace("% &%", ",")
        return p
    def detect_lng(Text:str):
        f = str(translator.detect(Text))
        c = f.replace("Detected(lang=", "")
        g = c.replace(")", "")
        r = g.replace(", confidence=", "==")
        i, u = r.split("==")
        return i
    def translate(Text:str, Dest:str, From:str="auto"):
        Text = str(str(Text).replace(",", "%&%"))
        if From == "auto": lng = using.detect_lng(Text)
        else:lng = From
        uncut = translator.translate(Text, src=lng, dest=Dest)
        resp = using.translation_cut(uncut)
        return resp
def Translate(Text:str, Dest:str, From:str='auto'):
    return using.translate(Text,Dest.lower(),From.lower())